/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;

import jonelo.NumericalChameleon.*;
import java.math.*;
import java.util.*;

/**
 * Cluster for Radix-Calculation 2 to 36
 */

public class ClusterBases35 extends ClusterObject {

    public static final int INTERNAL=35;
    public final static String CODE="RADIX35";

    private Vector v;

    public static String
      RADIX="Radix",
      DUAL="Dual",
      OCTAL="Octal",
      DECIMAL="Decimal",
      HEX="Hexadecimal",
      INVALID="Source is invalid";

    /** Creates new ClusterBase35 */
    public ClusterBases35() {
        v = new Vector();
        String name = null;
        for (int i = 2; i < 37; i++)
        {
            name=RADIX+" "+i;
            if (i==2) name=name+" ("+DUAL+")"; else
            if (i==8) name=name+" ("+OCTAL+")"; else
            if (i==10) name=name+" ("+DECIMAL+")"; else
            if (i==16) name=name+" ("+HEX+")";

            v.add(new NcRecord(name,"int"));
        }
    }

    public Vector getSourceEntries() {
        return v;
    }

    public Vector getTargetEntries() {
        return v;
    }

    public int getSourceDefault() {
        return 8; // base 10 (index 10-2)
    }

    public int getTargetDefault() {
        return 14; // base 16 (index 16-2)
    }

    public int getPreferredPrecision()  {
        return -1;
    }

    public String getOutput(int s, int t) throws Exception
    {
        BigInteger big = null;
        try {
          big = new BigInteger(input,s+2);
        } catch (Exception e) {
            throw new Exception(INVALID);
        }
        return big.toString(t+2).toUpperCase();
    }

    public String addValue(BigDecimal plus, int sourceIndex) throws Exception
    {
        if ((input==null) || (input.equals(""))) input="0";
        BigDecimal big = new BigDecimal(getTransferValue(sourceIndex));
        big=big.add(plus);
        return setValue(big,sourceIndex);
    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception
    {
        setInput(big.toString());
        return getOutput(getSourceDefault(), sourceIndex);
    }

    public String getTransferValue(int s) throws Exception {
          return getOutput(s,getSourceDefault());
    }
}
