/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import jonelo.NumericalChameleon.*;
import jonelo.sugar.util.*;

import java.math.*;
import java.util.*;
import java.text.*;
public class ClusterCalendar extends ClusterObject {

    public final static int INTERNAL=18;
    public final static String CODE="CALENDAR";

    private final static int
      // source
      cDATE = 0,
      cDATEde = 1,
      cDATEus = 2,
      cDATEgb = 3,
      cDATEfr = 4,
      cDATEes = 5,
      cDATEbr = 6,
      cDATEmx = 7,

      // target
      cDAY_OF_WEEK=0,
      cDAY_OF_YEAR=1,
      cWEEK_OF_YEAR=2,
      cMONTH=3,
      cWEEK_OF_MONTH=4,
      cDAYS_IN_MONTH=5,
      cAGE=6,
      cAGE_IN_DAYS=7,
      cSTERNZEICHEN=8,
      cSUNSIGNS=9,
      cDATE_t=10,    FIRSTDATEINDEX=10,
      cDATEde_t=11,
      cDATEus_t=12,
      cDATEgb_t=13,
      cDATEfr_t=14,
      cDATEes_t=15,
      cDATEbr_t=16,
      cDATEmx_t=17;


    public static String
      DATE = "Date",
      LEAPYEAR = "Leapyear",
      WEEK_OF_YEAR = "Week of year",
      WEEK_OF_MONTH = "Week of month",
      DAYS_IN_MONTH="Days in month",
      DAY_OF_YEAR = "Day of year",
      DAY_OF_WEEK ="Day of week",
      MONTH = "Month",
      AGE = "Age",
      AGE_IN_DAYS = "Age in days",
      INVALID = "Source is invalid",
      YES = "yes",
      NO = "no";

    private static String[]
      STERNZEICHEN = { "Widder", "Stier", "Zwillinge", "Krebs", "Lwe", "Jungfrau",
       "Waage", "Skorpion", "Schtze", "Steinbock", "Wassermann", "Fische" },

      DEKADE = { " (1. Dekade)", " (2. Dekade)", " (3. Dekade)" },

      SUNSIGNS = { "Aries", "Taurus", "Gemini", "Cancer", "Leo", "Vigro",
      "Libra", "Scorpio", "Sagittarius", "Capricorn", "Aquarius", "Pisces" };


    private Vector vs,vt;

    public ClusterCalendar() {
        // source
        vs=new Vector();
        vs.add(new NcRecord(DATE+" (YYYYMMDD)","int"));
        vs.add(new NcRecord(DATE+" (DD.MM.YYYY)","de"));
        vs.add(new NcRecord(DATE+" (MM/DD/YYYY)","us"));

        vs.add(new NcRecord(DATE+" (DD/MM/YYYY)","gb"));
        vs.add(new NcRecord(DATE+" (DD/MM/YYYY)","fr"));
        vs.add(new NcRecord(DATE+" (DD/MM/YYYY)","es"));
        vs.add(new NcRecord(DATE+" (DD/MM/YYYY)","br"));
        vs.add(new NcRecord(DATE+" (DD/MM/YYYY)","mx"));

        // target
        vt=new Vector();
        vt.add(new NcRecord(DAY_OF_WEEK,"int"));
        vt.add(new NcRecord(DAY_OF_YEAR,"int"));
        vt.add(new NcRecord(WEEK_OF_YEAR,"int"));
        vt.add(new NcRecord(MONTH,"int"));
        vt.add(new NcRecord(WEEK_OF_MONTH,"int"));
        vt.add(new NcRecord(DAYS_IN_MONTH,"int"));
        vt.add(new NcRecord(AGE,"int"));
        vt.add(new NcRecord(AGE_IN_DAYS,"int"));
        vt.add(new NcRecord("Sternzeichen","de"));
        vt.add(new NcRecord("Sun signs","eng"));

        vt.add(new NcRecord(DATE+" (YYYYMMDD)","int"));
        vt.add(new NcRecord(DATE+" (DD.MM.YYYY)","de"));
        vt.add(new NcRecord(DATE+" (MM/DD/YYYY)","us"));
        vt.add(new NcRecord(DATE+" (DD/MM/YYYY)","gb"));
        vt.add(new NcRecord(DATE+" (DD/MM/YYYY)","fr"));
        vt.add(new NcRecord(DATE+" (DD/MM/YYYY)","es"));
        vt.add(new NcRecord(DATE+" (DD/MM/YYYY)","br"));
        vt.add(new NcRecord(DATE+" (DD/MM/YYYY)","mx"));
    }

    public Vector getSourceEntries()  {
        return vs;
    }

    public Vector getTargetEntries() {
        return vt;
    }

    public int getSourceDefault() {
        Locale l = Locale.getDefault();
        if (l.getLanguage().equals("de")) return cDATEde; else
        if (l.getLanguage().equals("fr")) return cDATEfr; else

        if (l.getLanguage().equals("en") && l.getCountry().equals("US")) return cDATEus; else
        if (l.getLanguage().equals("en")) return cDATEgb; else

        if (l.getLanguage().equals("pt") && l.getCountry().equals("BR")) return cDATEbr; else
        if (l.getLanguage().equals("pt")) return cDATEbr; else

        if (l.getLanguage().equals("es") && l.getCountry().equals("MX")) return cDATEmx; else
        if (l.getLanguage().equals("es")) return cDATEes;
        return cDATE;
    }

    public int getTargetDefault() {
        return cDAY_OF_WEEK;
    }

    public boolean isOneway() {
        return true;
    }
    /** get Output
     * @param s soure
     * @param t tareget
     * @throws Exception if there is a problem
     * @return String
     */
    public String getOutput(int s, int t) throws Exception
    {
        GregorianCalendar gc = new GregorianCalendar();

        int year = 0;
        int month = 0;
        int day = 0;
        gc.setLenient(false);

        switch (s)
        {
            // international
            case cDATE:   if (input.length() != 8) throw new Exception(INVALID);
                          try {
                            year = Integer.parseInt(input.substring(0,4));
                            month = Integer.parseInt(input.substring(4,6));
                            day = Integer.parseInt(input.substring(6,8));
                          } catch (Exception e) { throw new Exception(INVALID); }
                          break;
            // german
            case cDATEde: try {
                            day = Integer.parseInt(input.substring(0,2));
                            month = Integer.parseInt(input.substring(3,5));
                            year = Integer.parseInt(input.substring(6,input.length()));
                          } catch (Exception e) { throw new Exception(INVALID); }
                          break;
            // US-english
            case cDATEus: try {
                            month = Integer.parseInt(input.substring(0,2));
                            day = Integer.parseInt(input.substring(3,5));
                            year = Integer.parseInt(input.substring(6,input.length()));
                          } catch (Exception e) { throw new Exception(INVALID); }
                          break;
            case cDATEgb:
            case cDATEfr:
            case cDATEes:
            case cDATEbr:
            case cDATEmx: try {
                            day = Integer.parseInt(input.substring(0,2));
                            month = Integer.parseInt(input.substring(3,5));
                            year = Integer.parseInt(input.substring(6,input.length()));
                          } catch (Exception e) { throw new Exception (INVALID); }
                          break;
        }

        try {
          gc.set(year,month-1,day);
          // dummy get() to force an exception in case of an invalid date (lenient is false)
          gc.get(Calendar.DAY_OF_YEAR);
        } catch (Exception e) { throw new Exception(INVALID); }

        switch (t)
        {
            //case cDATE: return DateFormat.getDateInstance(DateFormat.FULL).format(gc.getTime());
            case cDATE_t: {
                           DecimalFormat digs2 = new DecimalFormat("00");
                           DecimalFormat digs4 = new DecimalFormat("0000");
                           StringBuffer sb = new StringBuffer(8);
                           sb.append(digs4.format(gc.get(Calendar.YEAR)));
                           sb.append(digs2.format(gc.get(Calendar.MONTH)+1));
                           sb.append(digs2.format(gc.get(Calendar.DATE)));
                           return sb.toString();
                        }
            case cDATEde_t: {
                           DecimalFormat digs2 = new DecimalFormat("00");
                           DecimalFormat digs4 = new DecimalFormat("0000");
                           StringBuffer sb = new StringBuffer(10);
                           sb.append(digs2.format(gc.get(Calendar.DATE)));
                           sb.append(".");
                           sb.append(digs2.format(gc.get(Calendar.MONTH)+1));
                           sb.append(".");
                           sb.append(digs4.format(gc.get(Calendar.YEAR)));
                           return sb.toString();
            }
            case cDATEus_t: {
                           DecimalFormat digs2 = new DecimalFormat("00");
                           DecimalFormat digs4 = new DecimalFormat("0000");
                           StringBuffer sb = new StringBuffer(10);
                           sb.append(digs2.format(gc.get(Calendar.MONTH)+1));
                           sb.append("/");
                           sb.append(digs2.format(gc.get(Calendar.DATE)));
                           sb.append("/");
                           sb.append(digs4.format(gc.get(Calendar.YEAR)));
                           return sb.toString();
            }

            case cDATEgb_t:
            case cDATEfr_t:
            case cDATEes_t:
            case cDATEbr_t:
            case cDATEmx_t: {
                           DecimalFormat digs2 = new DecimalFormat("00");
                           DecimalFormat digs4 = new DecimalFormat("0000");
                           StringBuffer sb = new StringBuffer(10);
                           sb.append(digs2.format(gc.get(Calendar.DATE)));
                           sb.append("/");
                           sb.append(digs2.format(gc.get(Calendar.MONTH)+1));
                           sb.append("/");
                           sb.append(digs4.format(gc.get(Calendar.YEAR)));
                           return sb.toString();
            }

            case cWEEK_OF_YEAR: return Integer.toString(gc.get(Calendar.WEEK_OF_YEAR));
            case cWEEK_OF_MONTH: return Integer.toString(gc.get(Calendar.WEEK_OF_MONTH));
            case cDAY_OF_YEAR: return Integer.toString(gc.get(Calendar.DAY_OF_YEAR));
            case cDAYS_IN_MONTH: return Integer.toString(gc.getActualMaximum(Calendar.DAY_OF_MONTH));

            case cDAY_OF_WEEK:
            {
                SimpleDateFormat formatter = new SimpleDateFormat("EEEE");
                return formatter.format(gc.getTime());
            }
                //return dayOfWeekAsString_en(gc.get(Calendar.DAY_OF_WEEK));
            case cMONTH:
            {
                SimpleDateFormat formatter = new SimpleDateFormat("MMMM");
                return formatter.format(gc.getTime());
                //  return Integer.toString(gc.get(Calendar.MONTH)+1);
            }
            case cAGE: {
                   Calendar today = Calendar.getInstance();
                   int age = today.get(Calendar.YEAR) - gc.get(Calendar.YEAR);
                   gc.add(Calendar.YEAR, age);
                   if (today.before(gc)) age--;
                   return Integer.toString(age);
                 }
            case cAGE_IN_DAYS: {
                Calendar today = Calendar.getInstance();
                return Integer.toString(ExtendedGregorianCalendar.getDaysBetween(gc,today));
            }
            case cSTERNZEICHEN: {
                int dat = (gc.get(Calendar.MONTH)+1)*100 + gc.get(Calendar.DATE);
                int code=0;
                int dekade=2;
                if ((dat >=  321) && (dat <=  420)) { code= 0;
                  if (dat <= 331) dekade=0; else
                  if (dat <= 401) dekade=1;
                } else // Widder
                if ((dat >=  421) && (dat <=  520)) { code= 1;
                  if (dat <= 430) dekade=0; else
                  if (dat <= 510) dekade=1;
                } else // Stier
                if ((dat >=  521) && (dat <=  621)) { code= 2;
                  if (dat <= 541) dekade=0; else
                  if (dat <= 611) dekade=1;
                } else // Zwillinge
                if ((dat >=  622) && (dat <=  722)) { code= 3;
                  if (dat <= 701) dekade=0; else
                  if (dat <= 712) dekade=1;
                } else // Krebs
                if ((dat >=  723) && (dat <=  823)) { code= 4;
                  if (dat <= 802) dekade=0; else
                  if (dat <= 812) dekade=1;
                } else // Lwe
                if ((dat >=  824) && (dat <=  923)) { code= 5;
                  if (dat <= 902) dekade=0; else
                  if (dat <= 912) dekade=1;
                } else // Jungfrau
                if ((dat >=  924) && (dat <= 1023)) { code= 6;
                  if (dat <= 1003) dekade=0; else
                  if (dat <= 1013) dekade=1;
                } else // Waage
                if ((dat >= 1024) && (dat <= 1122)) { code= 7;
                  if (dat <= 1102) dekade=0; else
                  if (dat <= 1112) dekade=1;
                } else // Skorpion
                if ((dat >= 1123) && (dat <= 1221)) { code= 8;
                  if (dat <= 1202) dekade=0; else
                  if (dat <= 1212) dekade=1;
                } else // Schtze
                if ((dat >= 1222) || (dat <=  120)) { code= 9;
                  if ((dat >= 1222) && (dat <= 1231)) dekade=0; else
                  if (dat <= 110) dekade=1;
                } else // Steinbock
                if ((dat >=  121) && (dat <=  219)) { code=10;
                  if (dat <= 130) dekade=0; else
                  if (dat <= 209) dekade=1;
                } else // Wassermann
                if ((dat >=  220) && (dat <=  320)) { code=11;
                  if (dat <= 229) dekade=0; else
                  if (dat <= 310) dekade=1;
                } // Fische
                return STERNZEICHEN[code]+DEKADE[dekade];
            }

            case cSUNSIGNS: {
                int dat = (gc.get(Calendar.MONTH)+1)*100 + gc.get(Calendar.DATE);
                int code=0;

                if ((dat >=  321) && (dat <=  419)) code= 0; else // Aries
                if ((dat >=  420) && (dat <=  520)) code= 1; else // Taurus
                if ((dat >=  521) && (dat <=  621)) code= 2; else // Gemini
                if ((dat >=  622) && (dat <=  722)) code= 3; else // Cancer
                if ((dat >=  723) && (dat <=  822)) code= 4; else // Leo
                if ((dat >=  823) && (dat <=  922)) code= 5; else // Vigro
                if ((dat >=  923) && (dat <= 1023)) code= 6; else // Libra
                if ((dat >= 1024) && (dat <= 1122)) code= 7; else // Skorpio
                if ((dat >= 1123) && (dat <= 1221)) code= 8; else // Sagittarius
                if ((dat >= 1222) || (dat <=  119)) code= 9; else // Capricorn
                if ((dat >=  120) && (dat <=  218)) code=10; else // Aquarius
                if ((dat >=  219) && (dat <=  320)) code=11;      // Pisces
                return SUNSIGNS[code];
            }
        }
        return "";
    }


    public String getInitialValue() {
       Locale l = Locale.getDefault();
       String f = null;

       if (l.getLanguage().equals("en") && l.getCountry().equals("US")) f="MM/dd/yyyy"; else
       if (l.getLanguage().equals("en")) f="dd/MM/yyyy"; else
       if (l.getLanguage().equals("de")) f="dd.MM.yyyy"; else
       if (l.getLanguage().equals("fr")) f="dd/MM/yyyy"; else
       if (l.getLanguage().equals("pt")) f="dd/MM/yyyy"; else
       if (l.getLanguage().equals("es")) f="dd/MM/yyyy"; else
       f="yyyyMMdd";

       SimpleDateFormat formatter = new SimpleDateFormat(f);
       return formatter.format(new Date());
    }

    public static String getInternationalTodayValue() {
       SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
       return formatter.format(new Date());
    }

    public String getTransferValue(int s) throws Exception {
        return getOutput(s,cDATE_t);
    }

    public String addValue(BigDecimal plus, int sourceIndex) throws Exception {

        String temp = getTransferValue(sourceIndex);
        int year = Integer.parseInt(temp.substring(0,4));
        int month = Integer.parseInt(temp.substring(4,6));
        int day = Integer.parseInt(temp.substring(6,8));

        int count = Integer.parseInt(plus.toString());
        GregorianCalendar gc = new GregorianCalendar(year,month-1,day);
        gc.add(Calendar.DATE,count);
        // lenient is true per default

        DecimalFormat digs2 = new DecimalFormat("00");
        DecimalFormat digs4 = new DecimalFormat("0000");
        StringBuffer sb = new StringBuffer(8);
        sb.append(digs4.format(gc.get(Calendar.YEAR)));
        sb.append(digs2.format(gc.get(Calendar.MONTH)+1));
        sb.append(digs2.format(gc.get(Calendar.DATE)));

        setInput(sb.toString());

        return getOutput(cDATE,sourceIndex+FIRSTDATEINDEX);
    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception {

        String temp = big.toString();
        if (temp.length() != 8) throw new Exception(INVALID);
        int year = Integer.parseInt(temp.substring(0,4));
        int month = Integer.parseInt(temp.substring(4,6));
        int day = Integer.parseInt(temp.substring(6,8));

        GregorianCalendar gc = new GregorianCalendar();
        gc.setLenient(false);

        gc.set(year,month-1,day);
        // dummy get() to force an exception in case of an invalid date (lenient is false)
        gc.get(Calendar.DAY_OF_YEAR);

        // if date is ok
        setInput(temp.substring(0,8));
        return getOutput(cDATE,sourceIndex+FIRSTDATEINDEX);
    }



}
