/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import jonelo.NumericalChameleon.*;
import java.math.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;


public class ClusterColor extends ClusterObject {

    public static final int INTERNAL=2;
    public final static String CODE="COLOR";

    private Vector v;

    public static String
    DECIMAL="Decimal",
    INVALID="Source is invalid";


    public ClusterColor() {
        v = new Vector();
        v.add(new NcRecord("RGB (dec)","int"));
        v.add(new NcRecord("RRGGBB (hex)","int"));
        v.add(new NcRecord("R, G, B (dec)","int"));
        v.add(new NcRecord("H, S, B (dec)","int"));
    }

    public Vector getSourceEntries() {
        return v;
    }

    public Vector getTargetEntries() {
        return v;
    }

    public int getSourceDefault() {
        return 0;
    }

    public int getTargetDefault() {
        return 2;
    }

    public int getPreferredPrecision()  {
        return -1;
    }

    public String getOutput(int s, int t) throws Exception {
        long l = 0;
        if (s==0) {
            try {
                l = Long.valueOf(input,10).longValue();
                if ((l < 0) || (l > 0x00FFFFFFL)) throw new Exception();
            } catch (Exception e) {
                throw new Exception(INVALID);
            }
        }

        if (s==1) {
            try {
                l = Long.valueOf(input,16).longValue();
                if ((l < 0) || (l > 0x00FFFFFFL)) throw new Exception();
            } catch (Exception e) {
                throw new Exception(INVALID);
            }
        }

        if (s==2) // r, g, b
        {
            try {
                Vector v = new Vector();
                Integer i = null;
                StringTokenizer st = new StringTokenizer(input,",");
                while (st.hasMoreTokens()) {
                    i = Integer.valueOf(st.nextToken().trim(),10);
                    v.add(i);
                }
                if (v.size() != 3) throw new Exception();

                Color c = new Color(
                ((Integer)v.get(0)).intValue(),
                ((Integer)v.get(1)).intValue(),
                ((Integer)v.get(2)).intValue());
                l = c.getRGB() & 0x00ffffffL;

            } catch (Exception e) {
                throw new Exception(INVALID);
            }
        }

        if (s==3) // h, s, b
        {
            try {
                Vector v = new Vector();
                float f[] = new float[3];
                StringTokenizer st = new StringTokenizer(input,",");
                // hue
                if (st.hasMoreTokens()) {
                   f[0] = Float.valueOf(st.nextToken().trim()).floatValue();
                   if (f[0] > 359) throw new Exception(INVALID+" (H > 359)");
                   else f[0] /= 359;
                } else throw new Exception(INVALID);

                // saturation
                if (st.hasMoreTokens()) {
                   f[1] = Float.valueOf(st.nextToken().trim()).floatValue();
                   if (f[1] > 100) throw new Exception(INVALID+" (S > 100)");
                   else f[1] /= 100;
                } else throw new Exception(INVALID);

                // brightness
                if (st.hasMoreTokens()) {
                   f[2] = Float.valueOf(st.nextToken().trim()).floatValue();
                   if (f[2] > 100) throw new Exception(INVALID+" (B > 100)");
                   else f[2] /= 100;
                } else throw new Exception(INVALID);

                if (st.hasMoreTokens()) throw new Exception();

                int color = Color.HSBtoRGB(f[0],f[1],f[2]);
                l = color & 0x00ffffffL;

            } catch (Exception e) {
                throw new Exception(e.getMessage());
            }
        }

        if (t==0) {
            return Long.toString(l);
        } else

        if (t==1) {
            String temp = Long.toHexString( l & 0x00ffffffL ).toUpperCase();
            // add a leading zero
            if (temp.length()==5) return "0"+temp;
            return temp;
        } else

        if (t==2) {
            Color c = new Color((int)l);
            return c.getRed()+", "+c.getGreen()+", "+c.getBlue();
        } else

        if (t==3) {
            Color c = new Color((int)l);
            float f[] = Color.RGBtoHSB(c.getRed(),c.getGreen(),c.getBlue(),null);
            return Math.round(f[0]*359)+", "+Math.round(f[1]*100)+", "+Math.round(f[2]*100);
        }

        return null;
    }

    public String addValue(BigDecimal plus, int sourceIndex) throws Exception {
        if ((input==null) || (input.equals(""))) {
            if (sourceIndex==2 || sourceIndex==3)
            return "0, 0, 1"; else return "1";
        }
        String temp = getTransferValue(sourceIndex);
        BigDecimal big = new BigDecimal(temp);
        big=big.add(plus);
        return setValue(big,sourceIndex);
    }

    public String getTransferValue(int s) throws Exception {
        return getOutput(s,getTransferIndex());
    }

    private int getTransferIndex() {
        return 0;
    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception {
        setInput(big.toString());
        return getOutput(getTransferIndex(),sourceIndex);
    }

}
