/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import java.util.*;
import java.math.*;

abstract public class ClusterObject {

    String input;
    boolean oneway, scientificNotationSupported, scientific;
    int precision, preferredPrecision;
    int sourceDefault, targetDefault;

    /** Creates new ClusterObject */
    public ClusterObject() {
        oneway=false;
        scientificNotationSupported=false;
        scientific=false;
        precision=-1;
        preferredPrecision=-1;
        sourceDefault=0;
        targetDefault=0;
    }

    abstract public Vector getSourceEntries();

    abstract public Vector getTargetEntries();

    // get the default index for source
    public int getSourceDefault() {
        return sourceDefault;
    }

    // get the default index for target
    public int getTargetDefault() {
        return targetDefault;
    }

    // set the default index for source
    public void setSourceDefault(int i) {
        sourceDefault=i;
    }

    // set the default index for target
    public void setTargetDefault(int i) {
        targetDefault=i;
    }

    // set the input for the Object
    public void setInput(String s) {
        input=s;
    }

    // get the input of the Object
    public String getInput() {
        return input;
    }

    // get the output (the conversion)
    abstract public String getOutput(int s, int t) throws Exception;


    // get the precision
    public int getPrecision() {
        return precision;
    }

    // set the precision
    public void setPrecision(int precision) {
        this.precision=precision;
    }

    // get the preferred precision
    public int getPreferredPrecision() {
        return preferredPrecision;
    }

    // set the preferred precision
    public void setPreferredPrecision(int preferredPrecision) {
        this.preferredPrecision=preferredPrecision;
    }

    // is scientific format wanted?
    public void setScientific(boolean bool) {
        scientific=bool;
    }

    // set scientific format if wanted
    public boolean getScientific() {
        return scientific;
    }

    // is scientific format supported?
    public boolean isScientificSupported() {
        return scientificNotationSupported;
    }

    // set scientific format if supported
    public void setScientificSupported(boolean bool) {
        scientificNotationSupported=bool;
    }

    // is it a Oneway conversion?
    // it can disable the button switch
    public boolean isOneway() {
        return oneway;
    }

    // set the oneway flag
    public void setOneway(boolean b) {
        oneway=b;
    }

    public void play(int index) throws Exception {}

    public void stopPlaying() {}

    // do you also accept empty Strings for input?
    // e. g. ClusterDialCode
    public boolean acceptEmptyStrings() {
        return false;
    }

    // is there a value that should be set after
    // the category was changed? If this method returns
    // null, the cluster will be set with a transferValue
    // from previous category
    public String getInitialValue() {
        return null;
    }

    abstract public String getTransferValue(int index) throws Exception;

    abstract public String addValue (BigDecimal plus, int sourceindex) throws Exception;

    abstract public String setValue (BigDecimal big, int sourceindex) throws Exception;

}
