/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import jonelo.NumericalChameleon.RomanNumber.*;
import jonelo.NumericalChameleon.*;
import java.util.Vector;
import java.math.*;

public class ClusterRoman extends ClusterObject {

    public final static int INTERNAL=1;
    public final static String CODE="ROMAN";

    private Vector v;
    public static String DECIMAL="Decimal";
    public static String ROMAN="Roman";
    public static String INVALID="Value is invalid";

    /** Creates new ClusterRoman */
    public ClusterRoman() {
        v=new Vector();
        v.add(new NcRecord(DECIMAL,"int"));
        v.add(new NcRecord(ROMAN,"int"));
    }

    public Vector getSourceEntries() {
        return v;
    }

    public Vector getTargetEntries() {
        return v;
    }

    public int getSourceDefault() {
        return 0;
    }

    public int getTargetDefault() {
        return 1;
    }

    public String getOutput(int s, int t) throws Exception
    {
        RomanNumber rn = null;
        int i=0;
        // source
        if (s==0)  // source is decimal
        {
            try {
                i= Integer.parseInt(input);
            } catch (Exception e) {
                throw new Exception(INVALID);
            }
            rn = new RomanNumber(i);
        }
        else // source is roman
        {
            rn = new RomanNumber(input);
        }
        // target
        if (t==0) // target is decimal
        {
           return Long.toString(rn.toLong());
        } else // target is roman
        {
           return rn.toRoman();
        }
    }

    public String addValue(BigDecimal plus, int sourceIndex) throws Exception
    {
        if ((input==null) || (input.equals("")))
        {
            return (sourceIndex==0 ?  "1":"I");
        }
        String temp = getTransferValue(sourceIndex);
        BigDecimal big = new BigDecimal(temp);
        big=big.add(plus);
        return setValue(big,sourceIndex);
    }

    public String getTransferValue(int s) throws Exception {
        return getOutput(s,getSourceDefault());
    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception {
        setInput(big.toString());
        return getOutput(getSourceDefault(), sourceIndex);
    }

}
