/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann, 
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import jonelo.NumericalChameleon.SpokenNumbers.*;
import jonelo.NumericalChameleon.*;
import java.math.*;
import java.util.Vector;
import java.util.*;
import java.net.URL;

public class ClusterSpokenNumbers extends ClusterObject {
    public final static int INTERNAL = 21;
    public final static String CODE="SPOKENNUMBERS";        
    private Vector vsource, vtarget, modules;
    
    public static String 
      languageUS="US English",
      languageUK="UK English",
      languageFR="French",
      languageES="Spanish",
      languageDE="German",
      languageIT="Italian",
      languageFI="Finnish",
      languageIN_hindi="Hindi",
      languageIN_tamil="Tamil",
      languageHU="Hungarian",
      languageSE="Swedish",
      languageTR="Turkish",
      languagePL="Polish",
      languagePT="Portuguese",
      languageBR="Portuguese",
      languageRO="Romanian",
      languageRU="Russian",
      languageLAT="Latin",
      languageJP="Japanese",
      languageESP="Esperanto",
      languageKLINGON="Klingon", // [Rumaiym]
      languageMORSE="Morsecode",
      languageNL="Dutch",
      
      DECIMAL="Decimal",
      INVALID="Source is invalid";
    
    SpokenNumber currentModule;
    
    /** Creates new ClusterSpokenNumbers */
    public ClusterSpokenNumbers() {
       setOneway(true);
       modules = new Vector();
       
       // source
       vsource = new Vector();
       vsource.add(new NcRecord(DECIMAL,"int"));
             
       // target
       vtarget = new Vector();   
       // Java provides a method called getDisplayLanguage() in class Locale, but
       // 1. only if you have a localized JRE and
       // 2. only for some languages like english, german, italian,
       // but not swedish or portuguese or other interesting languages
       // So we use our own language resource files ;-)

       vtarget.add(new NcRecord(languageUS, "us"));
       modules.add(new USEnglishNumber());

       vtarget.add(new NcRecord(languageUK, "gb"));
       modules.add(new UKEnglishNumber());

       vtarget.add(new NcRecord(languageFR, "fr"));
       modules.add(new FrenchNumber());

       vtarget.add(new NcRecord(languageES, "es"));
       modules.add(new SpanishNumber());

       vtarget.add(new NcRecord(languageDE, "de"));
       modules.add(new GermanNumber());

       vtarget.add(new NcRecord(languageIT, "it"));
       modules.add(new ItalianNumber());
              
       vtarget.add(new NcRecord(languageFI, "fi"));
       modules.add(new FinnishNumber());
       
       vtarget.add(new NcRecord(languageIN_hindi, "in"));
       modules.add(new IndianHindiNumber());
              
       vtarget.add(new NcRecord(languageIN_tamil, "in"));
       modules.add(new IndianTamilNumber());
       
       vtarget.add(new NcRecord(languageHU, "hu"));
       modules.add(new HungarianNumber());
       
       vtarget.add(new NcRecord(languageSE, "se"));
       modules.add(new SwedishNumber());

       vtarget.add(new NcRecord(languageTR, "tr"));
       modules.add(new TurkishNumber());
       
       vtarget.add(new NcRecord(languagePL, "pl"));
       modules.add(new PolishNumber());
       
       vtarget.add(new NcRecord(languageBR, "br"));
       modules.add(new PortugueseBrazilNumber());

       vtarget.add(new NcRecord(languagePT, "pt"));
       modules.add(new PortugueseNumber());

       vtarget.add(new NcRecord(languageRO, "ro"));
       modules.add(new RomanianNumber());

       vtarget.add(new NcRecord(languageRU, "ru"));
       modules.add(new RussianNumber());

       vtarget.add(new NcRecord(languageJP, "jp"));       
       modules.add(new JapaneseNumber());
/*
       will be available in NC 1.7
       vtarget.add(new NcRecord(languageNL, "nl"));
       modules.add(new DutchNumber());
*/       
       vtarget.add(new NcRecord(languageESP, "esp"));
       modules.add(new EsperantoNumber());
              
       vtarget.add(new NcRecord(languageLAT, "int"));
       modules.add(new LatinNumber());

       vtarget.add(new NcRecord(languageMORSE, "int"));
       modules.add(new MorseCodeNumber());

       vtarget.add(new NcRecord(languageKLINGON, "startrek"));
       modules.add(new KlingonNumber());
    }
    
    public Vector getSourceEntries()  {        
        return vsource;
    }
    
    public Vector getTargetEntries() {
        return vtarget;
    }    
    
    public void play(int index) throws Exception {
        if (!((input==null) || input.equals(""))) {
          currentModule = (SpokenNumber)modules.get(index);
          String dir = currentModule.getSoundDir();          
          if (dir != null) {
            URL url = getClass().getResource("/data/sounds/"+dir);
            if (url==null) throw new Exception("not supported");
          } else throw new Exception("not supported");
          currentModule.play();
        } 
    }
    
    public void stopPlaying() {
         if (currentModule!=null) currentModule.stopPlaying();
    }
            
    /** get Output
     * @param s soure
     * @param t tareget
     * @throws Exception if there is a problem
     * @return String
     */    
    public String getOutput(int s, int t) throws Exception {       
        // is input a valid number?
        BigInteger big = null;
        BigDecimal bigd = null;        
        try {
          bigd = new BigDecimal(input);
          big = bigd.toBigInteger();
        } catch (Exception e) {
            throw new Exception(INVALID);
        }   
        // s is always 0
        currentModule = (SpokenNumber)modules.get(t);
        currentModule.setNumber(big.toString());
        return currentModule.toString();        
    }    

    public String getTransferValue(int s) throws Exception {
        BigInteger big = null;
        BigDecimal bigd = null;
        try {
          bigd = new BigDecimal(input);
          big = bigd.toBigInteger();          
        } catch (Exception e) {
            throw new Exception(INVALID);
        }
        return big.toString();        
    }    

    public String addValue(BigDecimal plus, int sourceindex) throws Exception {                
        if ((input==null) || (input.equals(""))) input="0";

        BigInteger bigi = null;
        BigDecimal bigd = null;
        try {
          bigd = new BigDecimal(getInput());
          bigi = bigd.toBigInteger();
        } catch (Exception e) {
            throw new Exception(INVALID);
        }   

        bigi=bigi.add(plus.toBigInteger());
        return bigi.toString();   
    }

    public String setValue(BigDecimal big, int sourceindex) throws Exception {        
        setInput(big.toString());
        return input;
    }
    
}
