/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;
import jonelo.NumericalChameleon.*;
import java.math.*;
import java.util.*;
import java.text.*;
import java.io.*;

public class ClusterTimezones extends ClusterObject {

    public final static int INTERNAL=558;
    public final static String CODE="TIMEZONES";

    public static String
      INVALID = "Source is invalid",
      YES = "yes",
      NO = "no";

    int sourceDefault=0;
    int targetDefault=0;
    private Vector v;
    private Vector v2;
    private DateFormat dateFormat;
    private TimeZone defaultTimeZone;
    private final static String yyyyMMddHHmmss = "yyyyMMddHHmmss";

    public ClusterTimezones() {
        dateFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM);
        v = new Vector();
        v2= new Vector();
        String[] availableIDs = TimeZone.getAvailableIDs();
        Hashtable hash = new Hashtable();
        // 1.4+:  Boolean dummy = Boolean.valueOf(true);
        Boolean dummy = new Boolean("true");
        for (int i=0; i < availableIDs.length; i++)
           hash.put(availableIDs[i],dummy);
        try {
            // read file
            InputStream is = getClass().getResourceAsStream("/data/lists/timezones.list");
            BufferedReader br = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
            String thisLine;
            StringBuffer sb = new StringBuffer();
            StringTokenizer st = null;
            String to = null;
            String from = null;
            int index=0;
            defaultTimeZone = TimeZone.getDefault();
            String timeZoneDefault = defaultTimeZone.getID();
            while ((thisLine = br.readLine()) != null) {
                if (!thisLine.startsWith("#") && !thisLine.equals("")) {
                    st = new StringTokenizer(thisLine,":");
                    String countrycode = st.nextToken();
                    String timezonecode = st.nextToken();
                    if (hash.get(timezonecode)!=null) {
                      v2.add(timezonecode);
                      StringBuffer sbt = new StringBuffer(96);
                      sbt.append(timezonecode);
                      sbt.append(" (");
                      sbt.append(Main.iso3166.getString(countrycode));
                      sbt.append(")");
                      v.add(new NcRecord(sbt.toString(), countrycode));

                      if (timezonecode.equals(timeZoneDefault)) sourceDefault=index;
                      if (timezonecode.equals("America/New_York")) targetDefault=index;
                      index++;
                    } else {
                        System.out.println("Timezone "+timezonecode+" not found.");
                    }
                }
            }
            br.close();
        } catch (Exception e) { System.err.println(e);}
    }

    public Vector getSourceEntries()  {
        return v;
    }

    public Vector getTargetEntries() {
        return v;
    }

    public int getSourceDefault() {
        return sourceDefault;
    }

    public int getTargetDefault() {
        return targetDefault;
    }

    /** get Output
     * @param s soure
     * @param t tareget
     * @throws Exception if there is a problem
     * @return String
     */
    public String getOutput(int s, int t) throws Exception {

      /*
        see http://javaalmanac.com/egs/java.util/GetTimeOtherZone2.html
        There is a convenient setTimeZone() method in the Calendar object.
        However, it doesn't always return the correct results when used after
        a calendar field is set. This example demonstrates a more reliable
        way to convert a specific time from one time zone to another. It
        involves creating two Calendar instances and transfering the
        UTC (Coordinate Universal Time) from one to the other. The UTC is
        a representation of time and date that is independent of time zones.
      */

      // Create a Calendar object with the source time zone
      TimeZone fromTimeZone = TimeZone.getTimeZone((String)v2.elementAt(s));
      Calendar fromCal = new GregorianCalendar(fromTimeZone);
      dateFormat.setTimeZone(fromTimeZone);
      fromCal.setTime(dateFormat.parse(input));

      // Create an instance using target's time zone and set it with the UTC
      TimeZone toTimeZone = TimeZone.getTimeZone((String)v2.elementAt(t));
      Calendar toCal = new GregorianCalendar(toTimeZone);
      // setTimeInMillis is protected in 1.3 and public in 1.4
      // 1.4+: 
      // toCal.setTimeInMillis(fromCal.getTimeInMillis());
      // 1.3+:
      toCal.setTime(fromCal.getTime());

      dateFormat.setTimeZone(toTimeZone);
      return dateFormat.format(toCal.getTime());
    }


    public String getInitialValue() {
       dateFormat.setTimeZone(defaultTimeZone);
       return dateFormat.format(new Date());
    }

    public static String getInternationalTimeValue() {
       SimpleDateFormat formatter = new SimpleDateFormat(yyyyMMddHHmmss);
       return formatter.format(new Date());
    }    

    public String addValue(BigDecimal plus, int sourceIndex) throws Exception {
        // Create a Calendar object with source "time zone"
        TimeZone fromTimeZone = TimeZone.getTimeZone((String)v2.elementAt(sourceIndex));
        Calendar fromCal = new GregorianCalendar(fromTimeZone);
        
        // set the internal value "input" for the calendar
        dateFormat.setTimeZone(fromTimeZone);
        fromCal.setTime(dateFormat.parse(input));

        // add the value "plus" to the calendar object
        int count = Integer.parseInt(plus.toString());
        fromCal.add(Calendar.HOUR,count);

        // set the value internally and return it
        String temp = dateFormat.format(fromCal.getTime());
        setInput(temp);
        return (temp);

    }

    public String setValue(BigDecimal big, int sourceIndex) throws Exception {
   
        // Create a calendar object with the source-timezone
        TimeZone fromTimeZone = TimeZone.getTimeZone((String)v2.elementAt(sourceIndex));
        Calendar fromCal = new GregorianCalendar(fromTimeZone);
        
        // set the value "big" for the calendar
        DateFormat standardDateFormat = new SimpleDateFormat(yyyyMMddHHmmss);        
        fromCal.setTime(standardDateFormat.parse(big.toString()));

        // convert the "big" value in the form which is used internally
        dateFormat.setTimeZone(fromTimeZone);
        String temp = dateFormat.format(fromCal.getTime());
        setInput(temp);
        return (temp);
    }
    
    public String getTransferValue(int s) throws Exception {
        
        // Create a calendar object with the source-timezone
        TimeZone fromTimeZone = TimeZone.getTimeZone((String)v2.elementAt(s));
        Calendar fromCal = new GregorianCalendar(fromTimeZone);
        
        // set the internal value "input" for the calendar
        dateFormat.setTimeZone(fromTimeZone);
        fromCal.setTime(dateFormat.parse(input));
        
        // convert the "input" value in the form which can be used externally
        // (the standard decimal date format yyyyMMddHHmmss)
        DateFormat standardDateFormat = new SimpleDateFormat(yyyyMMddHHmmss);
        return standardDateFormat.format(fromCal.getTime());        
    }
    

}
