/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann, 
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.Clusters;

import jonelo.NumericalChameleon.*;
import java.math.*;
import java.util.*;
import jonelo.sugar.util.*;


public class ClusterUnicode extends ClusterObject {

    public static final int INTERNAL=3;
    public final static String CODE="UNICODE";
    
    private Vector v;    
    
    public static String 
      DECIMAL="Decimal",
      HEX="Hexadecimal",
      INVALID="Source is invalid";
    
    /** Creates new ClusterBase35 */
    public ClusterUnicode() {        
        v = new Vector();        
        v.add(new NcRecord(DECIMAL,"int"));
        v.add(new NcRecord(HEX,"int"));        
        v.add(new NcRecord("Unicode","int"));        
    }
            
    public Vector getSourceEntries() {
        return v;
    }
    
    public Vector getTargetEntries() {
        return v;
    }    
    
    public int getSourceDefault() {
        return 0; 
    }
    
    public int getTargetDefault() {
        return 2;
    }
    
    public int getPreferredPrecision()  {
        return -1;
    }
        
    public String getOutput(int s, int t) throws Exception
    {        
        long[] longArray = null;
        // convert to decimal values

        if (s==1) { // hexadecimal
          if ((input.length() > 4) && (input.length() % 4 != 0))
              throw new Exception(INVALID);
          
          try {
            if (input.length() <= 4) {
                longArray= new long[1];
                longArray[0]=Long.valueOf(input,16).longValue();
                if (longArray[0] < 0) throw new Exception();
            } else {              
                longArray=new long[input.length() / 4];
                for (int i=0; i < longArray.length; i++) {
                   longArray[i]=Long.valueOf(input.substring(i*4,(i+1)*4),16).longValue();
                }
            }
          } catch (Exception e) {
              throw new Exception(INVALID);
          }    
        } else

        if (s==2) { // unicode char
           if (input.length()==0) return "";
           longArray=new long[input.length()];
           for (int i=0; i < input.length(); i++) {
               longArray[i] = (int)input.charAt(i);
           }
        } else

        if (s==0) { // decimal
          try {
            if (input.length()==0) return "";
            int count = GeneralString.countChar(input, ',');
            if (count == 0) { 
              long l = Integer.valueOf(input,10).longValue();
              if ((l < 0) || (l > 0xFFFFl)) throw new Exception();                
              longArray=new long[1];
              longArray[0]=l;              
            } else {
              longArray=new long[count+1]; // 999,123 => count==1, e. g. 2 tokens
              StringTokenizer st = new StringTokenizer(input,","); 
              int i=0;
              while (st.hasMoreTokens()) {
                 String token = st.nextToken();
                 if (token.equals("")) { 
                     longArray[i]=0;
                 } else {
                   long l = Integer.valueOf(token,10).longValue();
                   if ((l < 0) || (l > 0xFFFFl)) throw new Exception();                   
                   longArray[i]=l;                   
                 }
                 i++;
              }
            }            
            
          } catch (Exception e) {
              throw new Exception(INVALID);
          }         
        }

        // now we have the decimal value,
        // convert it to the target
        
        if (t==1) { // hexadecimal
            if (longArray.length == 1)
                return Long.toString(longArray[0],16).toUpperCase();
            
            StringBuffer sb = new StringBuffer(longArray.length*4);            
            for (int i=0; i < longArray.length; i++) {
              sb.append(hexformat(longArray[i],4).toUpperCase());
            }
            return sb.toString();
        } else

        if (t==2) { // unicode
            StringBuffer sb = new StringBuffer(longArray.length);
            for (int i=0; i < longArray.length; i++) {
              sb.append((char)longArray[i]);
            }
            return sb.toString();
        } else
        
        if (t==0) { // decimal
            StringBuffer sb = new StringBuffer(longArray.length*4); // "999,"
            for (int i=0; i < longArray.length; i++) {
                sb.append(longArray[i]);
                sb.append(",");
            }
            sb.deleteCharAt(sb.toString().length()-1);
            return sb.toString();
        }
        return null;
    }    
    
    public static String hexformat(long value, int nibbles) {
        StringBuffer sb = new StringBuffer(Long.toHexString(value));
        while (sb.length() < nibbles) sb.insert(0,'0');
        return sb.toString();
    }
    
    public String addValue(BigDecimal plus, int sourceIndex) throws Exception
    {
        if ((input==null) || (input.equals(""))) 
        {
            return (sourceIndex==2 ?  new Character((char)1).toString():"1");            
        }        
        String temp = getTransferValue(sourceIndex);
        BigDecimal big = new BigDecimal(temp); 
        big=big.add(plus);
        return setValue(big,sourceIndex);   
    }

    public String getTransferValue(int s) throws Exception {        
        return getOutput(s,getSourceDefault());
    }
    
    public String setValue(BigDecimal big, int sourceIndex) throws Exception {
        setInput(big.toString());
        return getOutput(getSourceDefault(), sourceIndex);        
    }
    
}
