/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.ExchangeRates;
import jonelo.NumericalChameleon.Main;
import jonelo.sugar.io.GeneralIO;
import jonelo.sugar.util.Counter;
import jonelo.sugar.util.GeneralString;
import java.io.*;
import java.util.*;
import javax.swing.*;
import gnu.regexp.*;
import java.math.*;


public class CurrencyConverter {

    private static boolean debug=false;
    private static BigDecimal _1 = new BigDecimal("1");

    public static void setDebug(boolean flag) {
        debug=flag;
    }

    // filenameFilter = filter, e. g. pacific filter
    // filenameRates = e. g. rates.html
    public CurrencyConverter(String filenameFilter, String filenameRates, Counter current, JTextArea textArea, ExchangeRates exchangeRates)
    throws Exception
    {
      try {
        // read the iso 4217 resource bundle
        ResourceBundle iso4217 = ResourceBundle.getBundle("data.lang.iso4217");

        // useful variables
        int count; // each entry
        int real; // real currency codes
        StringBuffer sb= new StringBuffer();
        String temp = null;

        // init
        textArea.setCaretPosition(0);
        textArea.setText("");

        // read properties of the filter and store props in vars
        Properties props = new Properties();
        props.load(getClass().getResourceAsStream(filenameFilter));

        String date = props.getProperty("date","");
        String before_iso = props.getProperty("before_iso","");
        String after_iso = props.getProperty("after_iso","");
        String before_iso_inv = props.getProperty("before_iso_inv","");
        String after_iso_inv = props.getProperty("after_iso_inv","");
        char decimalSeparator=props.getProperty("decimalSeparator"," ").charAt(0);
        char decimalPoint=props.getProperty("decimalPoint"," ").charAt(0);
        String logicString = props.getProperty("logic","false");
        exchangeRates.setLogic(logicString.equals("true"));
        String base = props.getProperty("base","");

        // store valuepos
        temp=props.getProperty("valuepos","1");
        int valuepos=1;
        try {
          valuepos = Integer.valueOf(temp).intValue();
        } catch (Exception e) {}

        // store syntax
        temp = props.getProperty("syntax","perl5");
        RESyntax syntax = null;
        if (temp.equals("perl5")) syntax = RESyntax.RE_SYNTAX_PERL5; else
        if (temp.equals("perl4")) syntax = RESyntax.RE_SYNTAX_PERL4; else
        syntax = RESyntax.RE_SYNTAX_PERL5;

        // read the rates file you have downloaded
        FileInputStream fin = new FileInputStream (filenameRates);
        BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));

        String allText = null;
        String thisLine;
        while ((thisLine=myInput.readLine()) != null) {
          sb.append(thisLine);
          sb.append("\n");
        }
        allText = sb.toString();
        myInput.close();
        fin.close();

        // read iso file
        count=0;
        real=0;

        // use iso4217 file
        String isocode = null;
        String description = null;
        StringTokenizer stringTokenizer = null;
        InputStream is = getClass().getResourceAsStream("/data/lists/iso4217.list");
        myInput = new BufferedReader (new InputStreamReader(is));
        thisLine=null;

        RE reg = null;
        REMatchEnumeration en = null;

        // search for date in rates file
        // and store it with exchangeRates.setDate()
        if (date.equals(""))
        {
            temp = "n/a";
        } else
        if (date.equals("today")) // date was set to "today"
        {
           temp= new Date().toString();
        } else
        {
            try {
               reg = new RE(date, RE.REG_ICASE, syntax);
            } catch (Exception e) { System.err.println(e); }
            en = reg.getMatchEnumeration(allText);

            if (en.hasMoreMatches())
            {
              temp=en.nextMatch().toString(1);
            } else {
              temp="n/a";
            }
        }
        exchangeRates.setDate(temp);

        // write date to textArea and stdout
        String out = "Exchange rates from: "+temp+"\n";
        if (debug) System.out.print(out);
        textArea.append(out);


        String value = null;
        String last_isocode = "---";
        String flag = "";
        String sourcetarget = "";
        String truefalse = "";
        boolean found = false;
        Counter counter2 = new Counter(0);
        String expr_template = before_iso+"   "+after_iso;
        String expr_template_inv = before_iso_inv+"   "+after_iso_inv;
        String expr = null;
        String country=null;
        String StringBuffer=null;

        // process, read iso4217.list
        while ((thisLine = myInput.readLine()) != null)
        {
           if (!thisLine.startsWith("#") && !thisLine.equals(""))
           {
              // user cancellation?
              current.inc();
              counter2.inc();
              if (current.get() != counter2.get()) throw new Exception("User cancellation.");

              // parse each line
              // isocode:description:flag:truefalse:sourcetarget
              stringTokenizer = new StringTokenizer(thisLine,":");
              isocode= stringTokenizer.nextToken();

              description = stringTokenizer.nextToken();

              try {
                description = GeneralString.encodeUnicode(iso4217.getString(isocode));
              } catch (MissingResourceException mre) {
                 description = "UNKNOWN";
              }

              sourcetarget="";
              truefalse="";
              flag="";
              if (stringTokenizer.hasMoreTokens())
              {
                flag = stringTokenizer.nextToken().toLowerCase();
                if (stringTokenizer.hasMoreTokens())
                {
                   truefalse=stringTokenizer.nextToken().toLowerCase();
                    if (stringTokenizer.hasMoreTokens())
                    {
                       sourcetarget=stringTokenizer.nextToken().toLowerCase();
                    }
                }
              }

              if (isocode.equals(base)) {
                  last_isocode=base;
                  value="1";
                  found=true;
              }

              if (!isocode.equals(last_isocode))
              {
                  expr = GeneralString.replaceString(expr_template,before_iso.length(),isocode);
                  // this is better performance than
                  // expr = before_iso + isocode + after_iso;

                  // search for expression (search for decimal value)
                  try {
                     reg = new RE(expr, RE.REG_ICASE | RE.REG_MULTILINE, syntax);
                  } catch (Exception e) { System.err.println(e.toString()+": "+expr); }

                  en = reg.getMatchEnumeration(allText);
                  if (en.hasMoreElements()) {

                    value = en.nextMatch().toString(valuepos);
                    // modify the value you found
                    // remove separator
                    value = GeneralString.removeChar (value,decimalSeparator);
                    if (decimalPoint != '.')
                    {
                       // replace decimalPoint with .
                       value = GeneralString.replaceChar (value,decimalPoint,'.');
                    }
                    // output
                    found = true;
                    real++;

                    count++;
                    country=Main.iso3166.getString(flag);
                    if (country!=null) country=GeneralString.encodeUnicode(country);
                    if (country==null) country=""; else country=" - "+country;
                    out = isocode+" ("+description+country+"):"+value+":"+flag+":"+truefalse+":"+sourcetarget;
                    exchangeRates.add(out);
                  } else
                  { // bloomberg fix (or any other website which mixes value and 1/values)

                     if (!(before_iso_inv.equals("") && (after_iso_inv.equals(""))))
                     {
                          expr = GeneralString.replaceString(expr_template_inv,before_iso_inv.length(),isocode);
                         // search for expression (search for decimal value)
                          try {
                            reg = new RE(expr, RE.REG_ICASE | RE.REG_MULTILINE, syntax);
                          } catch (Exception e) { System.err.println(e.toString()+": "+expr); }

                          en = reg.getMatchEnumeration(allText);
                          if (en.hasMoreElements()) {

                             value = en.nextMatch().toString(valuepos);
                             // modify the value you found
                             // remove separator
                             value = GeneralString.removeChar (value,decimalSeparator);
                             if (decimalPoint != '.')
                             {
                                // replace decimalPoint with .
                                value = GeneralString.replaceChar (value,decimalPoint,'.');
                             }
                             // calculate value = 1/value
                             BigDecimal bd = new BigDecimal(value);
                             bd=_1.divide(bd,10,BigDecimal.ROUND_HALF_UP);
                             value=bd.toString();

                             // output
                             found = true;
                             real++;

                             count++;
                             country=Main.iso3166.getString(flag);
                             if (country!=null) country=GeneralString.encodeUnicode(country);
                             if (country==null) country=""; else country=" - "+country;
                             out = isocode+" ("+description+country+"):"+value+":"+flag+":"+truefalse+":"+sourcetarget;
                             exchangeRates.add(out);
                          } else {
                            found = false;
                            out = isocode + " not found";
                          }
                     } else {
                       found = false;
                       out = isocode + " not found";
                     }
                  } // end bloomberg fix
                  last_isocode = isocode;
              } else // value already parsed
              {
                  if (found) {
                    count++;
                    country=Main.iso3166.getString(flag);
                    if (country!=null) country=GeneralString.encodeUnicode(country);
                    if (country==null) country=""; else country=" - "+country;
                    sb = new StringBuffer(isocode.length()+description.length()+country.length()+value.length()+flag.length()+truefalse.length()+sourcetarget.length()+7);
                    sb.append(isocode).append(" (").append(description).append(country).append("):").append(value).append(":").append(flag).append(":").append(truefalse).append(":").append(sourcetarget);
                    // better than
                    // out = isocode+" ("+description+country+"):"+value+":"+flag+":"+truefalse+":"+sourcetarget;
                    out=sb.toString();
                    exchangeRates.add(out);
                  } else
                  out = isocode + " not found";
              }
              if (debug) System.out.println(out);
              textArea.append(out+"\n");
              textArea.setCaretPosition(textArea.getDocument().getLength());
           } // end if
        } // end while

        // last work
        myInput.close();
        fin.close();

        int tempi = current.get();
        if (tempi != counter2.get())
        {
           throw new Exception("User cancellation.");
        }
        double d = (count*100.0/tempi);
        java.math.BigDecimal bd = new java.math.BigDecimal(d);
        bd = bd.setScale(2, java.math.BigDecimal.ROUND_HALF_UP);
        d = bd.doubleValue();
        out = count+" of "+tempi+" exchange rates found ("+d+" %), ["+real+" unique]\n";

        if (debug) System.out.print(out);
        textArea.append(out);
        if (count == 0) exchangeRates.clear();

    } catch (Exception e) {
      exchangeRates.clear();
      System.err.println(e.toString());
      throw e;
    }
  }



}