/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
/**
  Uses a SwingWorker to perform a time-consuming (and utterly fake) task.
 */

package jonelo.NumericalChameleon.ExchangeRates;
import java.util.Properties;
import java.util.Vector;
import javax.swing.*;
import java.io.*;
import jonelo.sugar.gui.SwingWorker;
import jonelo.sugar.util.Counter;

public class LongTask {
    private int lengthOfTask;
    private String statMessage;
    private String ratesFilename;
    private String includeFilename;
    private Counter counter=null;
    private JTextArea textArea = null;
    private JButton button = null;

    private ExchangeRates exchangeRates;

    LongTask(ExchangeRates exchangeRates) {
        //Compute length of task...
      this.exchangeRates=exchangeRates;
      lengthOfTask=0;
      try {
        FileInputStream fin = new FileInputStream ("../data/lists/iso4217.list");
        BufferedReader myInput = new BufferedReader(new InputStreamReader(fin));
        String thisLine = null;
        while ((thisLine = myInput.readLine()) != null) {
           if (!thisLine.startsWith("#") && !thisLine.equals(""))
               lengthOfTask++;
        }
      } catch (Exception e) {}
      counter = new Counter();
    }

    void setRatesFilename(String filename) {
        this.ratesFilename=filename;
    }

    void setIncludeFilename(String filename) {
        this.includeFilename = filename;
    }


    void setTextArea(JTextArea textArea) {
        this.textArea=textArea;
    }

    void setButton(JButton button) {
        this.button=button;
    }

    /**
     * Called to start the task.
     */
    void go() {
        counter.set(0);
        final SwingWorker worker = new SwingWorker() {
            public Object construct() {
                return new ActualTask();
            }
        };
        worker.start();
    }

    /**
     * Called from ProgressBarDemo to find out how much work needs
     * to be done.
     */
    int getLengthOfTask() {
        return lengthOfTask;
    }

    /**
     * Called from ProgressBarDemo to find out how much has been done.
     */
    int getCurrent() {
        return counter.get();
    }

    void stop() {
        counter.set(lengthOfTask);
    }

    /**
     * Called from ProgressBarDemo to find out if the task has completed.
     */
    boolean done() {
        if (counter.get() >= lengthOfTask)
            return true;
        else
            return false;
    }

    String getMessage() {
        return statMessage;
    }


    /**
     * The actual long running task.  This runs in a SwingWorker thread.
     */
    class ActualTask {

       ActualTask () {
       try {
          // do something      exchangeRatesUpdateVector.get(index));
          CurrencyConverter cc = new CurrencyConverter(ratesFilename, includeFilename, counter, textArea, exchangeRates);
          if (exchangeRates.success()) button.setEnabled(true);
        } catch (Exception e) {
            System.err.println(e.toString());
            System.err.println("ERROR: exchange rate list corrupted.");
            textArea.append(e.toString());
        }
        }
    }
}
