/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.RomanNumber;
import jonelo.sugar.util.GeneralString;

public class RomanNumber
{
  public static final int MIN_VALUE = 1;
  public static final int MAX_VALUE = 5000;

  private String sourceString=null;
  private long sourceLong=0;

  public static String
    ERROR_INVALIDCHAR="The roman value is invalid (only M, D, C, L, X, V and I are allowed).",
    ERROR_ISEMPTY="The roman value may not be empty",
    ERROR_CHARONLYONCE="The character {0} is allowed only once in a roman number",
    ERROR_CHAR3TIMES="It is prohibited to use the character {0} more than 3 times in a sequence",
    ERROR_RANGE="The decimal value must be between 1 and 5000 for roman calculations";

  public RomanNumber(String roman) throws RomanException
  {
    setString(roman);
  }

  public RomanNumber(long myLong) throws RomanException
  {
    setLong(myLong);
  }

  private int wandle(char c) throws RomanException
  {
     switch (c )
     {
       case 'M': return 1000;
       case 'D': return  500;
       case 'C': return  100;
       case 'L': return   50;
       case 'X': return   10;
       case 'V': return    5;
       case 'I': return    1;
       default:  throw new RomanException(ERROR_INVALIDCHAR);
     }
  }

  public void setString(String myString) throws RomanException
  {
    sourceString=myString.toUpperCase();
    try {
      sourceLong = convert2Long();
    } catch (RomanException e)
    {
      sourceLong=0;
      throw new RomanException(e.getMessage());
    }
  }

  public void setLong(long myLong) throws RomanException
  {
    sourceLong=myLong;
    try {
      sourceString = convert2Roman();
    } catch (RomanException e)
    {
      sourceString = null;
      throw new RomanException(e.getMessage());
    }
  }

  public long toLong()
  {
    return sourceLong;
  }

  public String toRoman()
  {
    return sourceString;
  }

   private long convert2Long() throws RomanException
   {
      if (sourceString.equals(""))
      throw new RomanException(ERROR_ISEMPTY);
      if (GeneralString.countChar(sourceString,'V') > 1)
      throw new RomanException(GeneralString.message(ERROR_CHARONLYONCE,'V'));
      if (GeneralString.countChar(sourceString,'L') > 1)
      throw new RomanException(GeneralString.message(ERROR_CHARONLYONCE,'L'));
      if (GeneralString.countChar(sourceString,'D') > 1)
      throw new RomanException(GeneralString.message(ERROR_CHARONLYONCE,'D'));

      if (sourceString.indexOf("IIII") >= 0)
      throw new RomanException(GeneralString.message(ERROR_CHAR3TIMES,'I'));
      if (sourceString.indexOf("XXXX") >= 0)
      throw new RomanException(GeneralString.message(ERROR_CHAR3TIMES,'X'));
      if (sourceString.indexOf("CCCC") >= 0)
      throw new RomanException(GeneralString.message(ERROR_CHAR3TIMES,'C'));

      int i, len=sourceString.length();
      long sum;

      // letztes Zeichen umwandeln
      try{
        sum=(long)wandle(sourceString.charAt(len-1));

        for(i = (len-1); i >= 1; i--)
        {
           long temp=wandle(sourceString.charAt(i-1));
           if (wandle(sourceString.charAt(i)) <= temp)
           sum=sum+temp; else sum=sum-temp;
        }
      } catch (RomanException e)
      { throw new RomanException(e.getMessage());}
      return sum;
   }

   private String convert2Roman() throws RomanException
   {
     StringBuffer buf = new StringBuffer("");
     long value=sourceLong;
     if ((value <=0) || (value > 5000)) throw new RomanException(ERROR_RANGE);
     while (value >= 1000) { buf.append('M');  value = value - 1000;}
     if (value >= 900)     { buf.append("CM"); value = value - 900; }
     if (value >= 500)     { buf.append('D');  value = value - 500; }
     if (value >= 400)     { buf.append("CD"); value = value - 400; }
     while (value >= 100)  { buf.append('C');  value = value - 100; }
     if (value >= 90)      { buf.append("XC"); value = value - 90; }
     if (value >= 50)      { buf.append('L');  value = value - 50; }
     if (value >= 40)      { buf.append("XL"); value = value - 40; }
     while (value >= 10)   { buf.append('X');  value = value - 10; }
     if (value >= 9)       { buf.append("IX"); value = value - 9; }
     if (value >= 5)       { buf.append('V');  value = value - 5; }
     if (value >= 4)       { buf.append("IV"); value = value - 4; }
     while (value > 0)     { buf.append('I');  value = value - 1; }
     return buf.toString();
   }

}