/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/

package jonelo.NumericalChameleon.SpokenNumbers;
import javax.sound.sampled.*;
import java.io.*;


import java.util.*;

public class AudioThread extends Thread {

    /** Creates a new instance of AudioThread */
    private Vector syllables = null;
    private String dir=null;
    private String suffix=null;
    private Properties props=null;
    boolean playing=false;    

    public AudioThread(Vector syllables, String dir) {
        // you have to copy the syllables!
        this.syllables=new Vector();
        for (int i=0; i < syllables.size(); i++)
            this.syllables.add(syllables.get(i));
        this.dir=dir;
        loadProperties();
    }

    public void run() {
      for (int i = 0; i < syllables.size(); i++) {
        if (syllables.elementAt(i).equals("-")) continue;
        if (isInterrupted()) break;
        // System.out.println(syllables.elementAt(i).toString()+"  "+isInterrupted());
        playAndWait("/data/sounds/"+dir+"/"+syllables.elementAt(i).toString().toLowerCase().trim()+props.get("format"));
      }
    }

    private void loadProperties() {
        props = new Properties();
        props.setProperty("format",".wav");
        try{
            InputStream is = getClass().getResourceAsStream("/data/sounds/"+dir+"/info");
            if (is != null) props.load(is);
        }  catch (Exception e) {
            System.err.println(e);
        }
    }

  private void playAndWait(String sound) {
   try {
        // From file
        AudioInputStream stream = AudioSystem.getAudioInputStream(getClass().getResource(sound));

        // At present, ALAW and ULAW encodings must be converted
        // to PCM_SIGNED before it can be played
        AudioFormat format = stream.getFormat();
        if (format.getEncoding() != AudioFormat.Encoding.PCM_SIGNED) {
//            System.out.println("convert to PCM_SIGNED");
            format = new AudioFormat(
                    AudioFormat.Encoding.PCM_SIGNED,
                    format.getSampleRate(),
                    format.getSampleSizeInBits()*2,
                    format.getChannels(),
                    format.getFrameSize()*2,
                    format.getFrameRate(),
                    true);        // big endian
            stream = AudioSystem.getAudioInputStream(format, stream);
        }

        // Create the clip
        DataLine.Info info = new DataLine.Info(
            Clip.class, stream.getFormat()); //, ((int)stream.getFrameLength()*format.getFrameSize()));
        Clip clip = (Clip) AudioSystem.getLine(info);

        // This method does not return until the audio file is completely loaded
        clip.open(stream);

        // set volume
        FloatControl gainControl = (FloatControl)clip.getControl(FloatControl.Type.MASTER_GAIN);
        double gain = 1.0;    // number between 0 and 1 (loudest)
        float dB = (float)(Math.log(gain)/Math.log(10.0)*20.0);
        gainControl.setValue(dB);

        // Start playing
        clip.start();
       
        double durationInSecs = clip.getBufferSize() /
          (clip.getFormat().getFrameSize() * clip.getFormat().getFrameRate());

        // warten, bis der sound komplett gespielt wurde
        try {
          Thread.sleep((int)(durationInSecs*1000));
        } catch (InterruptedException e) {
            interrupt();
        }

      
/*
        // Create the clip
        DataLine.Info info = new DataLine.Info(
            Clip.class, stream.getFormat()); //, ((int)stream.getFrameLength()*format.getFrameSize()));
        Clip clip = (Clip) AudioSystem.getLine(info);

        // This method does not return until the audio file is completely loaded
        clip.open(stream);

        // set volume
        FloatControl gainControl = (FloatControl)clip.getControl(FloatControl.Type.MASTER_GAIN);
        double gain = 1.0;    // number between 0 and 1 (loudest)
        float dB = (float)(Math.log(gain)/Math.log(10.0)*20.0);
        gainControl.setValue(dB);

        // Start playing
        clip.start();
       
        double durationInSecs = clip.getBufferSize() /
          (clip.getFormat().getFrameSize() * clip.getFormat().getFrameRate());

        // warten, bis der sound komplett gespielt wurde
        try {
          Thread.sleep((int)(durationInSecs*1000));
        } catch (InterruptedException e) {
            interrupt();
        }
*/
    } catch (IOException e) {
        System.out.println(e);
    } catch (LineUnavailableException e) {
        System.out.println(e);
    } catch (UnsupportedAudioFileException e) {
        System.out.println(e);
    }
  }

}
