/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
public class FrenchNumber extends SpokenNumber {

  private final static String field[][] = {
   {"un", "deux","trois","quatre","cinq","six","sept","huit","neuf"},
   {"onze","douze", "treize","quatorze", "quinze", "seize", "dix-sept", "dix-huit", "dix-neuf"},
   {"dix", "vingt","trente","quarante","cinquante","soixante","soixante-dix", "quatre-vingts", "quatre-vingt-dix"}
  },

  ET    = " et ",
  DASH  = "-",
  NULL  = "zro",
  MINUS = "moins";

  public FrenchNumber() {
      super();
  }

  public FrenchNumber(long number) throws Exception {
      super(number);
  }

  public FrenchNumber(String number) throws Exception {
     super (number);
  }


  protected int getSupportedDigits() {
	  return 2;
  }

  public String getSoundDir() {
	  return "french";
  }

  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {
    number2digits();
    if (number.charAt(0) == '-')
    {
      number=number.substring(1);
      syllables.addElement(MINUS+" ");
    }

    if (number.equals("0")) syllables.addElement(NULL);
    else {
      decode(0);
    }
  }

  private void decode(int index)
  {
    int x = (digits[index+1]*10)+digits[index];
    if (x < 10) {  // 0-9
       syllables.addElement(field[0][x-1]);
    } else
    if (x == 10) { // 10
        syllables.addElement(field[2][0]);
    } else
    if (x < 20) { // 11-19
       syllables.addElement(field[1][x-11]);
    } else
    if (x < 70) { // 20-69
       syllables.addElement(field[2][digits[index+1]-1]);
       if (digits[index] > 0)
       {
           if (digits[index] == 1)
                syllables.addElement(ET);
           else syllables.addElement(DASH);
           syllables.addElement(field[0][digits[index]-1]);
       }
    } else
    if (x < 80) { // 70-79 (70=60+10, 79=60+19)
        if (digits[index]==0) // 70
            syllables.add(field[2][6]); // soixante-dix
        else
        {
           syllables.add(field[2][5]); // soixante
           if (digits[index]==1)
                syllables.add(ET);
           else syllables.add(DASH);
           syllables.addElement(field[1][digits[index]-1]);
        }
    } else
    if (x < 90) { // 80-89
        if (digits[index]==0)
            syllables.add(field[2][7]);
        else
        {
            syllables.add(field[0][3]); // quatre
            syllables.add(DASH);
            syllables.add(field[2][1]); // vingt
            syllables.add(DASH);
            syllables.add(field[0][digits[index]-1]);
        }
    } else
    { // 90-99
        if (digits[index]==0)
            syllables.add(field[2][8]);
        else
        {
            syllables.add(field[0][3]); // quatre
            syllables.add(DASH);
            syllables.add(field[2][1]); // vingt
            syllables.add(DASH);
            syllables.add(field[1][digits[index]-1]);
        }
    }
  }


  /**
   * returns number in french words
   * @param number the number to be converted
   */
  public static String toString(long number) throws Exception
  {
    return (new FrenchNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new FrenchNumber(number)).toString();
  }

}
