/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
public class GermanNumber extends SpokenNumber
{
  private final static String field[][] = {
   {"ein","zwei","drei","vier","fnf","sechs","sieben","acht","neun"},
   {"elf","zwlf","dreizehn","vierzehn","fnfzehn","sechzehn","siebzehn","achtzehn","neunzehn"},
   {"zehn","zwanzig","dreissig","vierzig","fnfzig","sechzig","siebzig","achtzig","neunzig"}
  },

  amount[][] = {
    { "tausend",         "tausend" },
    { " Million ",       " Millionen " },
    { " Milliarde ",     " Milliarden "},
    { " Billion ",       " Billionen "},
    { " Billiarde ",     " Billiarden "},
    { " Trillion ",      " Trillionen "},
    { " Trilliarde ",    " Trilliarden " },
    { " Quadrillion ",   " Quadrillionen "},
    { " Quadrilliarde ", " Quadrilliarden "},
    { " Quintillion ",   " Quintillionen " },
    { " Quintilliarde ", " Quintilliarden "},
    { " Sextillion ",    " Sextillionen "},
    { " Sextilliarde ",  " Sextilliarden "},
    { " Septillion ",    " Septillionen "},
    { " Septilliarde ",  " Septilliarden "},
    { " Oktillion ",     " Oktillionen "},
    { " Oktilliarde ",   " Oktilliarden "},
    { " Nonillion ",     " Nonillionen "},
    { " Nonilliarde ",   " Nonilliarden "},
    { " Dezillion ",     " Dezillionen "},
    { " Dezilliarde ",   " Dezilliarden "},
    { " Undezillion ",   " Undezillionen "},
    { " Undezilliarde ", " Undezilliarden "},
    { " Duodezillion ",  " Duodezillionen "},
    { " Duodezilliarde "," Duodezilliarden "},
    { " Tredizillion ",  " Tredizillionen "},
    { " Tredizilliarde "," Tredizilliarden "},
    { " Quattuordezillion "," Quattuordezillionen "},
    { " Quattuordezilliarde "," Quattuordezilliarden "},
    { " Quindezillion ",  " Quindezillionen "},
    { " Quindezilliarde ", " Quindezilliarden "}
  },

  NULL            = "null",
  MINUS           = "minus",
  EINS            = "eins",
  EINE            = "eine",
  EIN             = "ein",
  UND             = "und",
  HUNDERT         = "hundert";

  public GermanNumber() {
      super();
  }

  public GermanNumber(long number) throws Exception {
      super(number);
  }

  public GermanNumber(String number) throws Exception {
     super (number);
  }


  protected int getSupportedDigits() {
	  return 96;
  }

  public String getSoundDir() {
	  return "german";
  }


  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {
    number2digits();
    if (number.charAt(0) == '-')
    {
      number=number.substring(1);
      syllables.addElement(MINUS+" ");
    }

    // die Zahlentriple abarbeiten
    if (number.equals("0")) syllables.addElement(NULL); else
    {
      int counter=amount.length-1;
      for (int i=MAX_DIGITS-3; i > 0; i-=3)
      {
        xtriple (i, i==3 ? EIN:EINE, amount[counter][0], amount[counter][1]);
        counter--;
      }
      triple (0, EINS);
    }
  }

  private void triple(int index, String smallestNumber)
  {
    if (digits[index+2] > 0)
    {
      syllables.addElement(field[0][digits[index+2]-1]);
      syllables.addElement(HUNDERT);
    }

    if ((digits[index+1] == 0) && (digits[index] == 1))        // XXXX01
    {
      syllables.addElement(smallestNumber);                    // ein, eins, eine (je nachdem)
    } else
    if ((digits[index+1] == 1) && (digits[index] > 0))         // XXXX11
    {
      syllables.addElement(field[1][digits[index]-1]);   // elf, zwlf, ...
    } else
    if ((digits[index+1] == 1) && (digits[index] == 0))        // XXXX10
    {
      syllables.addElement(field[2][digits[index+1]-1]);   // zehn, zwanzig, ...
    } else
    if (digits[index+1] != 1)                                   // XXXX2X oder XXXX0X
    {
      if (digits[index+0] > 0)                                  // XXXX21
      {
        syllables.addElement(field[0][digits[index]-1]); // ein, zwei, drei, ...
      }
      if (digits[index+1] > 0)                                  // XXXX2X
      {
        if (digits[index+0] > 0)                                // XXXX21
        {
          syllables.addElement(UND);                            // und
        }
        syllables.addElement(field[2][digits[index+1]-1]); // zehn, zwanzig,...
      }
    }
  }

  // index = Begin des Tripels
  // minimum = String, der die kleinste Zahl des Tripels kennzeichnet (z. B. EINE Million)
  // singular = z. B. Million
  // plural = z. B. Millionen
  private void xtriple(int index, String minimum, String singular, String plural)
  {
    if ((digits[index]+digits[index+1]+digits[index+2]) > 0 )
    {
      triple(index,minimum);
      int sum = digits[index] + digits[index+1]*10 + digits[index+2]*100;
      if (sum >= 2)
      {
        syllables.addElement(plural);
      } else
      {
        syllables.addElement(singular);
      }
    }
  }

  /**
   * gibt Nummer in deutschen Worten zurck.
   * @param number die zu konvertierende Zahl
   */
  public static String toString(long number) throws Exception
  {
    return (new GermanNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new GermanNumber(number)).toString();
  }

}
