/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
/**
 * Source: Apa Guide Indien, Polyglott, 1. Auflage 1999 (0-9)
 *
 */

public class IndianHindiNumber extends SpokenNumber {
  /*
   * Some numbers have similarity in pronunciation, but
   * there is no clear rule for numbers in Hindi.
  */
  private final static String field[] =
   { "shunya",   "ek",        "do",        "teen",        "char",        "paanch",     "chah",        "saat",       "aath",        "nau",        // 0-9
     "dus",      "gyaaruh",   "baaruh",    "tayruh",      "chauduh",     "pundruh",    "soluh",       "satruh",     "atthaaruh",   "unnees",     // 10-19
     "bees",     "ikkees",    "baaees",    "tayees",      "chaubees",    "puchchees",  "chhubbees",   "sattaaees",  "atthaaees",   "unatees",    // 20-29
     "tees",     "ikattees",  "battees",   "tayntees",    "chauntees",   "payntees",   "chchattees",  "sayntees",   "aratees",     "untaalees",  // 30-39
     "chaalees", "iktaalees", "bayaalees", "tayntaalees", "chouaalees",  "payntaalees","chhiyaatees", "sayntaalees","artaalees",   "unchaas",    // 40-49
     "puchaas",  "ikyaawun",  "baawun",    "tirpun",      "chouwun",     "pachpun",    "chhappun",    "sattaawun",  "atthhaawun",  "unsuth",     // 50-59
     "saath",    "iksath",    "baasuth",   "tirsath",     "chaunsath",   "paynsath",   "chhiyaasath", "sursath",    "arsath",      "unhattar",   // 60-69
     "suttar",   "ikhuttar",  "buhuttar",  "tihuttar",    "chauhuttar",  "puchuhuttar","chhihuttar",  "sutuhuttar", "athuhuttar",  "unyaasee",   // 70-79
     "ussee",    "ikyaasee",  "buyaasee",  "tiraasee",    "chauraasee",  "puchaasee",  "chhiyaasee",  "suttaasee",  "utthaasee",   "nuwaasee",   // 80-89
     "nubbay",   "ikyaanuway","baanuway",  "tiraanuway",  "chauraanuway","puchaanuway","chhiyaanuway","suttaanuway","utthhaanuway","ninyaanuway" // 90-99
   },
   MINUS="-";

  /** Creates new IndianHindi */
   public IndianHindiNumber() {
      super();
  }

  public IndianHindiNumber(long number) throws Exception {
      super(number);
  }

  public IndianHindiNumber(String number) throws Exception {
     super (number);
  }


  protected int getSupportedDigits() {
	  return 2;
  }

  public String getSoundDir() {
	  return "hindi";
  }


  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {
    number2digits();
    if (number.charAt(0) == '-')
    {
      number=number.substring(1);
      syllables.addElement(MINUS+" ");
    }
    syllables.addElement(field[ (digits[1]*10)+digits[0] ]);
  }

  /**
   * gibt Nummer in Hindi Worten zurck.
   * @param number die zu konvertierende Zahl
   */
  public static String toString(long number) throws Exception
  {
    return (new IndianHindiNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new IndianHindiNumber(number)).toString();
  }

}

