/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
public class LatinNumber extends SpokenNumber
{
  private final static String field[][] =
  {
   { "unus","duo","tres","quattuor","quinque","sex","septem","octo","novem" },
   { "undecim", "duodecim", "tredecim", "quattuordecim", "quindecim", "sedecim", "septendecim", "duodeviginti", "undeviginti" },
   { "decem", "viginti", "triginta", "quadraginta", "quinquaginta", "sexaginta", "septuaginta", "octoginta", "nonaginta" }
  },

  NULL       = "nihil",
  MINUS      = "-";

  public LatinNumber() {
      super();
  }

  public LatinNumber(long number) throws Exception {
      super(number);
  }

  public LatinNumber(String number) throws Exception {
     super(number);
  }


  protected int getSupportedDigits() {
	  return 2;
  }

  public String getSoundDir() {
	  return "latin";
  }

  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {
    number2digits();
    if (number.charAt(0) == '-')
    {
       number=number.substring(1);
       syllables.addElement(MINUS+" ");
    }

    // die Zahlentriple abarbeiten
    if (number.equals("0")) syllables.addElement(NULL); else
    {
      decode(0);
    }
  }

  private void decode(int index)
  {
    int x = (digits[index+1]*10)+digits[index];
    if (x < 10) {  // 1-9
       syllables.addElement(field[0][x-1]);
    } else
    if ((x > 10) && (x < 20)) { // 11-19
       syllables.addElement(field[1][x-11]);
    } else
    { // 10, 20-99
       syllables.addElement(field[2][digits[index+1]-1]);

       if (digits[index] > 0)
       {
           syllables.addElement(" "+field[0][digits[index]-1]);
       }
    }
  }




  /**
   * gibt Nummer in russischen Worten zurck.
   * @param number die zu konvertierende Zahl
   */
  public static String toString(long number) throws Exception
  {
    return (new LatinNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new LatinNumber(number)).toString();
  }

}
