/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 * @authors:
 * Johann N. Loefflmann, Germany in cooperation with
 * Joo Marcelo Pereira Alves, Brazil
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
public class PortugueseBrazilNumber extends SpokenNumber
{

  protected static final String
  line1[] = { "um ","dois ","trs ","quatro ","cinco ","seis ","sete ","oito ","nove " },
  line2[] = { "onze ", "doze ", "treze ", "quatorze ", "quinze ", "dezesseis ", "dezessete ", "dezoito ", "dezenove " },
  line3[] = { "dez ", "vinte ", "trinta ", "quarenta ", "cinqenta ", "sessenta ", "setenta ", "oitenta ", "noventa " },
  line4[] = { "cento ", "duzentos ", "trezentos ", "quatrocentos ", "quinhentos ", "seiscentos ", "setecentos ", "oitocentos ", "novecentos " };
  protected static final String fieldfix[][] = {
   line1, line2, line3, line4
  };

  protected String field[][];

  protected final static String
  amount[][] = {
    { "mil ",          "mil " },
    { "milho ",       "milhes " },
    { "bilho ",       "bilhes "},
    { "trilho ",      "trilhes "},
    { "quatrilho ",   "quatrilhes "},
    { "quintilho ",   "quintilhes "},
    { "sextilho ",    "sextilhes " },
    { "setilho ",     "setilhes "},
    { "octilho ",     "octilhes "},
    { "nonilho ",     "nonilhes " },
    { "decilho ",     "decilhes "}
  },

  NULL            = "zero",
  MINUS           = "menos ",
  AND             = "e ",
  HUNDERT         = "cem ";
  protected boolean setprefix=false;

  public PortugueseBrazilNumber() {
      super();
      field=fieldfix;
  }

  public PortugueseBrazilNumber(long number) throws Exception {
      super(number);
      field=fieldfix;
  }

  public PortugueseBrazilNumber(String number) throws Exception {
     super (number);
      field=fieldfix;
  }


  protected int getSupportedDigits() {
	  return 36;
  }

  public String getSoundDir() {
	  return "portuguese";
  }


  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {
    number2digits();
    setprefix=false;
    if (number.charAt(0) == '-')
    {
      number=number.substring(1);
      syllables.addElement(MINUS);
    }

    // die Zahlentriple abarbeiten
    if (number.equals("0")) syllables.addElement(NULL); else
    {
      int counter=amount.length-1;
      for (int i=MAX_DIGITS-3; i > 0; i-=3)
      {
        xtriple (i, amount[counter][0], amount[counter][1]);
        counter--;
      }
      xtriple (0, null, null);
    }
  }


  protected void triple(int index)
  {
    if (digits[index+2] > 0)
    {
      if((digits[index+2] == 1) && (digits[index+1] == 0 && digits[index] == 0))
      {
         syllables.addElement(HUNDERT);
      } else
      {
         syllables.addElement(field[3][digits[index+2]-1]);
         if ((digits[index]+digits[index+1]) != 0) syllables.addElement(AND);
      }
    }

    if ((digits[index+1] == 1) && (digits[index] > 0))         // XXXX11
    {
      syllables.addElement(field[1][digits[index]-1]);   // elf, zwlf, ...
    } else
    if ((digits[index+1] >= 1) && (digits[index] == 0))        // XXXX10
    {
          syllables.addElement(field[2][digits[index+1]-1]);   // zehn, zwanzig, ...
    } else                                                 // XXXX2X oder XXXX0X

      {
      if (digits[index+1] > 0)                                  // XXXX2X
      {
        syllables.addElement(field[2][digits[index+1]-1]); // zehn, zwanzig,...
        if (digits[index] > 0)                                // XXXX21
        {
          syllables.addElement(AND);                            // und
        }
      }

      if (digits[index] > 0)                                  // XXXX21
      {
        syllables.addElement(field[0][digits[index]-1]); // ein, zwei, drei, ...
      }
    }
  }


  protected boolean onlyZeros(int index)
  {
    boolean onlyZeros=true;
      for (int i=0; i<=index; i++)
      {
          if (digits[i] > 0) { onlyZeros=false; break; }
      }
    return onlyZeros;
  }
  // index = Begin des Tripels
  // singular = z. B. Million
  // plural = z. B. Millionen
  protected void xtriple(int index, String singular, String plural)
  {
    if ((digits[index]+digits[index+1]+digits[index+2]) > 0 )
    {
      /**
       * Rule for the prefix "e"
       * Between triples, there is "e" only when the third position is 0 or
       * when first and second are 0 in the triple on the right side AND
       * all the other digits in triples to the right must also be 0.
       *            _     _      __   _
       * with 'e': 1009, 1090, 1900, 1099
       * no 'e': 1909, 1990, 1999
       */
      if (
            setprefix &&
            (digits[index+2]==0 || (digits[index]==0 && digits[index+1]==0)) &&
            onlyZeros(index-1)
          )
      {
         syllables.addElement(AND);
      }

      int sum = digits[index] + digits[index+1]*10 + digits[index+2]*100;
      if (!((sum == 1) && (index==3))) // avoid "um mil"
      {
        triple(index);
      }
      setprefix=true;

      if (index != 0) // we dont add nulls (the last triple)
      {
        if (sum > 1)
        {
          syllables.addElement(plural);
        } else
        {
          syllables.addElement(singular);
        }
      }
    }
  }

  /**
   * gibt Nummer in portugiesischen Worten zurck.
   * @param number die zu konvertierende Zahl
   */
  public static String toString(long number) throws Exception
  {
    return (new PortugueseBrazilNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new PortugueseBrazilNumber(number)).toString();
  }

}
