/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
import java.lang.Thread;
import java.util.*;
import java.net.URL;
import jonelo.sugar.util.GeneralString;
import java.io.*;
import javax.sound.sampled.*;

abstract public class SpokenNumber
{
  protected int MAX_DIGITS=0;
  protected BigInteger
    MAX_VALUE = new BigInteger("0"),   // mu berschrieben werden
    MIN_VALUE = new BigInteger("0");   // mu berschrieben werden
  protected String
    SOUND_DIR = null;   // mu berschrieben werden
  public static String
    ERROR_TOOBIG = "Number is too big [maximum is (10^{0})-1]",
    ERROR_TOOSMALL = "Number is too small [minimum is -(10^{0})-1]";

  protected String number;
  protected Vector syllables = new Vector(); // Silbenvektor
  protected int digits[];
  protected AudioThread audioThread = null;

 /**
   * Default-Konstruktor.
   */
  public SpokenNumber()
  {
    init();
    try {
      setNumber(0);
    } catch (Exception e) {}
  }

  /**
   * Creates a new SpokenNumber
   * @param number die zu konvertierende Zahl
   * @exception Exception wenn number nicht zwischen MIN_VALUE und MAX_VALUE liegt
   */
  public SpokenNumber(long number) throws Exception
  {
    init();
    setNumber(number);
  }

  public SpokenNumber(String number) throws Exception
  {
    init();
    setNumber(number);
  }


  public void setNumber(long number) throws Exception
  {
    syllables.clear();
    checkNumber(number+"");
    this.number = Long.toString(number);
    //super.setNumber(number);
    convert2Syllables();
  }

  public void setNumber(String number) throws Exception
  {
    syllables.clear();
    checkNumber(number);
    //super.setNumber(number);
    this.number = number;
    convert2Syllables();
  }

  public String getNumber() {
    return number;
  }

  protected void checkNumber(String number) throws Exception
  {
    BigInteger big = new BigInteger(number);
    if (big.compareTo(MAX_VALUE) > 0)
    {
      throw new Exception(GeneralString.message(ERROR_TOOBIG,MAX_DIGITS));
    } else
    if (big.compareTo(MIN_VALUE) < 0)
    {
      throw new Exception (GeneralString.message(ERROR_TOOSMALL,MAX_DIGITS));
    }
  }

  /**
   * gibt die Nummber in Worten zurck.
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < syllables.size(); i++)
    {
      sb.append(syllables.elementAt(i).toString());
    }
    return sb.toString().trim();
  }

  abstract protected void convert2Syllables() throws Exception; // mu berschrieben werden

  protected void init() {
      MAX_DIGITS=getSupportedDigits();
      digits = new int[MAX_DIGITS];
      MAX_VALUE = new BigInteger(getMaximum(MAX_DIGITS));
      MIN_VALUE = new BigInteger("-"+MAX_VALUE);
  }

  // How many digits can we handle?
  abstract protected int getSupportedDigits();

  // How is the sound dir called?
  abstract public String getSoundDir();

  //public int getDigits() { return MAX_DIGITS; }


  public void play()
  {
    String dir = getSoundDir();
    if (dir!=null) {
        if (audioThread!=null)
        { // stop any old threads
            audioThread.interrupt();
            while (audioThread.isAlive())
            {
                // wait for a while
                try {
                  Thread.sleep(1);
                } catch (InterruptedException e) {}
            }
        }
        audioThread = new AudioThread(syllables,dir);
        audioThread.start();
    }
  }

  public void stopPlaying()
  {
     if (audioThread!= null)
         audioThread.interrupt();
  }


  protected void number2digits()
  {
      // absolut value verwenden!
      int offset;
      if (number.charAt(0) == '-') offset=1; else offset=0;
      for (int i=offset; i < number.length(); i++)
      {
          int digit=0;
          try {
              int start = (number.length()-i)-1;
              start+=offset;
             digit = Integer.parseInt(number.substring(start,start+1));
          } catch (Exception e) {}
          digits[i-offset]=digit;
      }
      for (int i=number.length(); i < MAX_DIGITS; i++)
          digits[i-offset]=0;

  }

  public static String getMaximum(int digits)
  {
    StringBuffer sb = new StringBuffer(digits);
    sb.setLength(digits);
    for (int i=0; i < digits; i++) sb.setCharAt(i,'9');
    return sb.toString();
  }


}