/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
public class TurkishNumber extends SpokenNumber {


  private final static String field[][] =
  {
    {"bir ","iki ","\u00E7 ","drt ","be\u015F ","alt\u0131 ","yedi ","sekiz ", "dokuz "},
    {"on bir","on iki","on \u00E7","on drt","on be\u015F","on alt\u0131","on yedi","on sekiz", "on dokuz"},
    {"on ","yirmi ","otuz ","k\u0131rk ","elli ","altm\u0131\u015F ","yetm\u0131\u015F ","seksen ","doksan "}
  },

  amount[] =
   {"bin ", "milyon ", "milyar ", "bilyon ", "bin bilyon", "trilyon ", "bin trilyon " },

   NULL = "s\u0131f\u0131r",
   MINUS = "eksi",
   HUNDRED = "yz ";

   public TurkishNumber() {
      super();
  }

  public TurkishNumber(long number) throws Exception {
      super(number);
  }

  public TurkishNumber(String number) throws Exception {
     super (number);
  }


  protected int getSupportedDigits() {
	  return 24;
  }

  public String getSoundDir() {
	  return "turkish";
  }


  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {

    number2digits();
    if (number.charAt(0) == '-')
    {
      number=number.substring(1);
      syllables.addElement(MINUS+" ");
    }

    // die Zahlentriple abarbeiten
    if (number.equals("0")) syllables.addElement(NULL); else
    {
      //  syllables.addElement(field[digits[0]-1]);
      int counter=amount.length-1;
      for (int i=MAX_DIGITS-3; i > 0; i-=3)
      {
        xtriple (i, amount[counter--]);
      }
      triple (0);
    }
  }

  /**
   * gibt Nummer in deutschen Worten zurck.
   * @param number die zu konvertierende Zahl
   */
  public static String toString(long number) throws Exception
  {
    return (new FinnishNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new FinnishNumber(number)).toString();
  }

  private void triple(int index)
  {
    if (digits[index+2] > 0)
    {
      // Modifikation fr "hundert"
      // falsch: bir yz (einhundert)
      // richtig: yz  (hundert), aber: iki yz (zweihundert)
      if (digits[index+2] > 1)
      syllables.addElement(field[0][digits[index+2]-1]);

      syllables.addElement(HUNDRED);
    }

    if ((digits[index+1] == 1) && (digits[index] > 0))         // XXXX11
    {
      syllables.addElement(field[1][digits[index]-1]);   // elf, zwlf, ...
    } else
    if ((digits[index+1] >= 1) && (digits[index] == 0))        // XXXX10
    {
      syllables.addElement(field[2][digits[index+1]-1]);   // zehn, zwanzig, ...
    } else                                                 // XXXX2X oder XXXX0X
    {
      if (digits[index+1] > 0)                                  // XXXX2X
      {
        syllables.addElement(field[2][digits[index+1]-1]); // zehn, zwanzig,...
      }

      if (digits[index] > 0)                                  // XXXX21
      {
        syllables.addElement(field[0][digits[index]-1]); // ein, zwei, drei, ...
      }
    }
  }

  private void xtriple(int index, String singular)
  {
    if ((digits[index]+digits[index+1]+digits[index+2]) > 0 )
    {
        // modifikation fr "tausend"
        // falsch:  bir bin (eintausend)
        // richtig: bin (tausend), aber: iki bin (zweitausend)
       if (!((index==3) && (digits[3]==1) && (digits[4]==0) && (digits[5]==0)))
       triple(index);

       syllables.addElement(singular);
    }
  }
}

