/******************************************************************************
 *
 * Numerical Chameleon version 1.6.0 - conversion tool
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * e-Mail: jonelo@jonelo.de
 *
 *****************************************************************************/
package jonelo.NumericalChameleon.SpokenNumbers;
import java.math.*;
public class UKEnglishNumber extends SpokenNumber
{

  protected String amount[] = null;

  protected static final String field[][] =
  {
   { "one","two","three","four","five","six","seven","eight","nine" },
   { "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "nineteen" },
   { "ten", "twenty", "thirty", "forty", "fifty", "sixty", "seventy", "eighty", "ninety" }
  },

  ukamount[] =
  {"thousand",
   "million",          "thousand million",
   "billion",          "thousand billion",
   "trillion",         "thousand trillion",
   "quadrillion",      "thousand quadrillion",
   "quintillion",      "thousand quintillion",
   "sextillion",       "thousand sextillion",
   "septillion",       "thousand septillion",
   "octillion",        "thousand octillion",
   "nonillion",        "thousand nonillion",
   "decillion",        "thousand decillion",
   "undecillion",      "thousand undecillion",
   "duodecillion",     "thousand duodecillion",
   "tredecillion",     "thousand tredecillion",
   "quattuordecillion","thousand quattuordecillion",
   "quindecillion",    "thousand quindecillion"
  },

   NULL       = "zero",
   MINUS      = "minus",
   AND        = "and",
   HUNDRED    = "hundred",
   DASH       = "-";

  public UKEnglishNumber() {
      super();
      amount=ukamount;
  }

  public UKEnglishNumber(long number) throws Exception {
      super(number);
      amount=ukamount;
  }

  public UKEnglishNumber(String number) throws Exception {
     super(number);
      amount=ukamount;
  }


  protected int getSupportedDigits() {
	  return 96;
  }

  public String getSoundDir() {
	  return "uk_english";
  }


  /**
   * wird zum Konvertieren der Zahl in Silben bentigt,
   * Ergebnis wird in einen Vektor geschrieben.
   */
  protected void convert2Syllables() throws Exception
  {
    number2digits();
    if (number.charAt(0) == '-')
    {
       number=number.substring(1);
       syllables.addElement(MINUS);
    }

    // die Zahlentriple abarbeiten
    if (number.equals("0")) syllables.addElement(NULL); else
    {
      int counter=amount.length-1;
      for (int i=MAX_DIGITS-3; i > 0; i-=3) {
        xtriple (i, amount[counter--]);
      }
      triple (0);
    }
  }

  // bei Zahlen ber 100 steht vor Zehnern und Einern "and"
  protected boolean isGreaterThan100() {
     for (int i=2; i < MAX_DIGITS; i++) {
       if (digits[i] > 0) { return true; }
     }
     return false;
  }

  protected void triple(int index)
  {
    if (digits[index+2] > 0)
    {
      syllables.addElement(field[0][digits[index+2]-1]);
      syllables.addElement(HUNDRED);
    }

    if (
         (index==0)  // only valid for index==0
         && ((digits[0]+digits[1]) != 0)  // only if value > 0
         && isGreaterThan100() // and > 100
       )
    syllables.addElement(AND);

    if ((digits[index+1] == 1) && (digits[index] > 0))         // XXXX11
    {
      syllables.addElement(field[1][digits[index]-1]);   // elf, zwlf, ...
    } else
    if ((digits[index+1] >= 1) && (digits[index] == 0))        // XXXX10
    {
      syllables.addElement(field[2][digits[index+1]-1]);   // zehn, zwanzig, ...
    } else                                                 // XXXX2X oder XXXX0X
    {
      if (digits[index+1] > 0)                                  // XXXX2X
      {
        syllables.addElement(field[2][digits[index+1]-1]); // zehn, zwanzig,...
        if (digits[index] > 0)                                // XXXX21
        {
          syllables.addElement("-");                            // und
        }
      }

      if (digits[index] > 0)                                  // XXXX21
      {
        syllables.addElement(field[0][digits[index]-1]); // ein, zwei, drei, ...
      }
    }
  }

  protected void xtriple(int index, String singular)
  {
    if ((digits[index]+digits[index+1]+digits[index+2]) > 0 )
    {
       triple(index);
       syllables.addElement(singular);
    }
  }

  // add spaces between syllables
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    boolean wasDash=false;
    for (int i = 0; i < syllables.size(); i++)
    {
	  if (i>0) {
		  boolean isDash=syllables.elementAt(i).equals(DASH);
		  if (!wasDash && !isDash)
			sb.append(" ");
	      wasDash=isDash;
      }
      sb.append(syllables.elementAt(i).toString());
    }
    return sb.toString();
  }


  /**
   * gibt Nummer in UK-englischen Worten zurck.
   * @param number die zu konvertierende Zahl
   */
  public static String toString(long number) throws Exception
  {
    return (new UKEnglishNumber(number)).toString();
  }

  public static String toString(String number) throws Exception
  {
    return (new UKEnglishNumber(number)).toString();
  }

}
