/*
  Copyright (C) 2003-2004 Robert F. Beeger (robert at beeger dot net)
  Licensed under the Academic Free License version 2.0
  OSI Certified Open Source Software

  Visit http://squareness.sourceforge.net/ for new releases of Squareness Look And Feel and other
  skins from the Squareness series.
*/
package net.beeger.squareness.delegate;

import java.awt.Graphics;
import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicButtonUI;

import net.beeger.squareness.SquarenessConstants;
import net.beeger.squareness.util.SquarenessBorderFactory;
import net.beeger.squareness.util.SquarenessButtonPainter;
import net.beeger.squareness.util.SquarenessListenerFactory;

/**
 * The Squareness Button UI delegate.
 */
public class SquarenessButtonUI extends BasicButtonUI
{
  /**
   * Create the UI delegate for the given component
   * 
   * @param component The component for which to create the ui delegate
   * @return The created ui delegate
   */
  public static ComponentUI createUI (JComponent component)
  {
    return _buttonUI;
  }

  /**
   * Install the UI delegate for the given component
   * 
   * @param component The component for which to install the ui delegate.
   */
  public void installUI (JComponent component)
  {
    component.putClientProperty(SquarenessConstants.ROLLOVER_CLENT_PROPERTY_KEY, Boolean.FALSE);
    component.addMouseListener(SquarenessListenerFactory.getButtonRolloverMouseListener());
    super.installUI(component);
  }

  /**
   * Uninstall the UI delegate
   *
   * @param component The component from which to uninstall the UI delegate.
   */
  public void uninstallUI (JComponent component)
  {
    component.removeMouseListener(SquarenessListenerFactory.getButtonRolloverMouseListener());
    super.uninstallUI(component);
  }

  /**
   * Paint the component.
   * 
   * @param graphics  The graphics resource used to paint the component
   * @param component The component to paint.
   */
  public void paint (Graphics graphics, JComponent component)
  {
    if (!"yes".equals(component.getClientProperty(SquarenessConstants.CHECKED_FOR_JEDIT_ROLLOVER)))
    {
      component.putClientProperty(SquarenessConstants.CHECKED_FOR_JEDIT_ROLLOVER, "yes");
      boolean jEditRolloverButton = false;
      Class currentClass = component.getClass();
      String currentClassName = currentClass.getName();
      while (!jEditRolloverButton && currentClass != null
             && !(currentClassName.startsWith("java.") || currentClassName.startsWith("javax.")))
      {
        if ("org.gjt.sp.jedit.gui.RolloverButton".equals(currentClass.getName()))
        {
          jEditRolloverButton = true;
        }
        else
        {
          currentClass = currentClass.getSuperclass();
        }
      }
      if (jEditRolloverButton)
      {
        component.setBorder(SquarenessBorderFactory.getButtonRolloverBorder());
      }
    }

    SquarenessButtonPainter.paintButton(graphics, (AbstractButton) component);
    super.paint(graphics, component);
  }

  /**
   * One instance handles all buttons.
   */
  private static SquarenessButtonUI _buttonUI = new SquarenessButtonUI();
}

/*
  $Log: SquarenessButtonUI.java,v $
  Revision 1.5  2004/03/08 15:12:26  rbeeger
  Added and enhanced comments.
  Simplified code.

  Revision 1.4  2004/03/04 09:32:54  rbeeger
  Oprimized imports.

  Revision 1.3  2004/03/03 19:55:49  rbeeger
  Added special handling of jEdit RolloverButtons

  Revision 1.2  2004/02/14 19:12:34  rbeeger
  Updated copyright statement
  Added cvs log directive at the end of each file
  Optimized imports
  Reformated code

 */
